package dk.nsp.idsas.test.idptest;

import dk.nsp.test.idp.NspSoapClient;
import dk.nsp.test.idp.model.Identity;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.commons.io.IOUtils;
import org.w3c.dom.Document;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.net.URI;
import java.nio.charset.StandardCharsets;

public class ServiceCaller {

    public <RequestType, ResponseType> ResponseType call(Identity identity, RequestType request, JAXBContext jaxbContext, String action, URI uri, Class responseClass) throws JAXBException {
        Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
        StringWriter sw = new StringWriter();
        jaxbMarshaller.marshal(request, sw);

        try (NspSoapClient client = new NspSoapClient(this::log)) {
            NspSoapClient.NspSoapRequest soapRequest = null;
            JAXBElement<ResponseType> jaxbElement;

            if (identity != null) {
                soapRequest = client.request(uri, action).as(identity);
            }

            try (NspSoapClient.NspSoapResponse soapResponse = soapRequest.execute(sw.toString())) {
                String responseXml = IOUtils.toString(soapResponse.getResponse(), StandardCharsets.UTF_8);

                if (soapResponse.isFault()) {
                    throw new NspServiceException(responseXml);
                }

                SOAPMessage message = MessageFactory.newInstance().createMessage(null, new ByteArrayInputStream(responseXml.getBytes()));
                jaxbElement = (JAXBElement<ResponseType>) jaxbContext.createBinder().unmarshal(message.getSOAPBody().extractContentAsDocument(), responseClass);

                return jaxbElement.getValue();

            } catch (NspServiceException e) {
                throw e;
            }
        } catch (Exception e){
            throw new RuntimeException(e);
        }
    }

    public <RequestType, ResponseType> ResponseType callRoot(Identity identity, RequestType request, SOAPOperation action, URI uri, Class requestClass, Class responseClass) throws JAXBException {
        JAXBContext jaxbContext = JAXBContext.newInstance(requestClass, responseClass);

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new RuntimeException(e);
        }
        Document document = db.newDocument();
        Marshaller jaxbMarshaller = jaxbContext.createMarshaller();

        // XmlRoot element is missing
        JAXBElement<RequestType> jaxbRootElement =
                new JAXBElement<>(new QName(action.getSoapNamespace(), action.getRootElement()),
                        requestClass,
                        request);
        jaxbMarshaller.marshal(jaxbRootElement, document);

        try (NspSoapClient client = new NspSoapClient(this::log)) {
            NspSoapClient.NspSoapRequest soapRequest = null;
            JAXBElement<ResponseType> jaxbElement;

            if (identity != null) {
                String soapOperation = action.getSoapOperation();
                soapRequest = client.request(uri, soapOperation).as(identity);
            }

            try {
                NspSoapClient.NspSoapResponse soapResponse = soapRequest.execute(XmlUtil.node2String(document.getDocumentElement()));
                String responseXml = IOUtils.toString(soapResponse.getResponse(), StandardCharsets.UTF_8);

                if (soapResponse.isFault()) {
                    throw new NspServiceException(responseXml);
                }

                if (responseClass == Void.class) {
                    return null;
                }

                SOAPMessage message = MessageFactory.newInstance().createMessage(null, new ByteArrayInputStream(responseXml.getBytes()));
                jaxbElement = (JAXBElement<ResponseType>) jaxbContext.createBinder().unmarshal(message.getSOAPBody().extractContentAsDocument(), responseClass);

                return jaxbElement.getValue();

            } catch (NspServiceException e) {
                throw e;
            }
        } catch (Exception e){
            throw new RuntimeException(e);
        }
    }

    private void log(String xml) {
        if (Boolean.getBoolean(this.getClass().getSimpleName() + ".log")) {
            String method = "Unknown";
            for (StackTraceElement e : new Exception().fillInStackTrace().getStackTrace()) {
                if (e.getClassName().equals(getClass().getName())) {
                    method = e.getMethodName();
                }
            }
            System.out.println("[" + method + "]");
            System.out.println(xml.replace("\n", ""));
        }
    }

}