package dk.nsp.idsas.ws;

import dk.nsp.idsas.security.IdsasContext;
import dk.nsp.idsas.security.IdsasContextBuilder;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.ws.SLALoggingServletFilter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import static dk.nsp.idsas.setup.ServiceSetup.ANNOTATION_CONFIG_CONTEXT;

public abstract class IdsasAbstractWS {
	@Resource
	protected WebServiceContext context;

	@Autowired
	protected IdsasContextBuilder idsasContextBuilder;

	@PostConstruct
	public void init() {
		ServletContext servletContext = (ServletContext) context.getMessageContext().get(MessageContext.SERVLET_CONTEXT);
		Object o = servletContext.getAttribute(ANNOTATION_CONFIG_CONTEXT);
		((AnnotationConfigApplicationContext)o).getAutowireCapableBeanFactory().autowireBean(this);
	}

	protected SLALogItem getSlaLogItemFromContext() {
		ServletRequest servletRequest = (ServletRequest) context.getMessageContext().get(MessageContext.SERVLET_REQUEST);
		return (SLALogItem) servletRequest.getAttribute(SLALoggingServletFilter.CURRENT_LOGITEM_REQUEST_KEY);
	}

	protected IdsasContext getValidatedIdsasContext(String patientId) {
		try {
			return idsasContextBuilder.getValidatedContext(patientId);
		} catch (IdsasSecurityException e) {
			// TODO: Better exception handling
			throw new RuntimeException(e);
		}
	}

	// TODO: For testing purposes?
	public void setContext(WebServiceContext context) { this.context = context; }
}
