package dk.nsp.idsas.security;

import dk.nsp.idsas.security.data.TestUser;
import org.junit.Assert;
import org.junit.Test;

import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.security.data.SecurityContextData;

import java.util.Optional;

public class IdsasActorProviderTest {

    @Test
    public void matchSystembruger() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(SecurityContextData::new);
        IdsasActor actor = provider.getActor();
        Assert.assertEquals(actor.getActorType(), IdsasActor.IdsasActorType.SYSTEM);
    }

    @Test
    public void matchSundhedsfaglig() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> new SecurityContextData() {
            @Override
            public Optional<User> getActingUser() {
                return Optional.of(new TestUser() {
                    @Override
                    public UserType getUserType() {
                        return UserType.HealthcareProfessional;
                    }
                });
            }
        });
        IdsasActor actor = provider.getActor();
        Assert.assertEquals(actor.getActorType(), IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
    }

    @Test(expected = IdsasSecurityException.class)
    public void ingenMatchMedPrincipalUser() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> new SecurityContextData() {
            @Override
            public Optional<User> getPrincipalUser() {
                return Optional.of(new TestUser());
            }
        });
        provider.getActor();
    }

    @Test(expected = IdsasSecurityException.class)
    public void ingenMatchMedActingUser() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> new SecurityContextData() {
            @Override
            public Optional<User> getActingUser() {
                return Optional.of(new TestUser());
            }
        });
        provider.getActor();
    }
}
