package dk.nsp.idsas.security.data;

import dk.sds.nsp.security.SecurityContext;
import org.w3c.dom.Element;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;

public class SecurityContextData implements SecurityContext {

    private static AtomicReference<Ticket> globalTicket = new AtomicReference<>(new DevelopmentTicket());

    private static ThreadLocal<Ticket> threadTicket = new ThreadLocal<>();

    private static final class DevelopmentTicket implements Ticket {
        private final ZonedDateTime now;

        private DevelopmentTicket() {
            this.now = ZonedDateTime.now();
        }

        @Override
        public boolean isValid() {
            return true;
        }

        @Override
        public Federation getFederation() {
            return Federation.Development;
        }

        @Override
        public Optional<String> getAudience() {
            return Optional.of("https://fsk");
        }

        @Override
        public Optional<ZonedDateTime> getCreated() {
            return Optional.of(now.minus(1, ChronoUnit.MINUTES));
        }

        @Override
        public Optional<ZonedDateTime> getValidFrom() {
            return Optional.of(now.minus(1, ChronoUnit.MINUTES));
        }

        @Override
        public Optional<ZonedDateTime> getValidTo() {
            return Optional.of(now.plus(1, ChronoUnit.MINUTES));
        }

        @Override
        public Optional<String> createRequestWithContext(Element element, Element... elements) {
            return Optional.empty();
        }
    }



    @Override
    public Optional<Ticket> getTicket() {
        Ticket ticket = threadTicket.get();
        if (ticket == null) {
            ticket = globalTicket.get();
        }
        return Optional.of(ticket);
    }

    @Override
    public Optional<Message> getMessage() {
        return Optional.empty();
    }

    @Override
    public Optional<User> getActingUser() {
        return Optional.empty();
    }

    @Override
    public Optional<User> getPrincipalUser() {
        return Optional.empty();
    }

    @Override
    public Optional<Organisation> getOrganisation() {
        return Optional.of(new Organisation() {
            @Override
            public OrganisationIdentifierFormat getIdentifierFormat() {
                return OrganisationIdentifierFormat.CVR;
            }

            @Override
            public String getIdentifier() {
                return "33257872";
            }

            @Override
            public Optional<String> getName() {
                return Optional.of("Statens Serum Institut");
            }
        });
    }

    @Override
    public Optional<Client> getClient() {
        return Optional.empty();
    }
}