package dk.nsp.idsas.ws;

import dk.nsp.idsas.util.HealthIndicator;
import dk.nsp.idsas.util.HealthIndication;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

@WebServlet(name="HealthServlet", urlPatterns={"/status"})
public class HealthServlet extends AbstractServlet {
	private static final Logger logger = Logger.getLogger(HealthServlet.class);

	private static final long serialVersionUID = 1L;

	@Autowired
	private List<HealthIndicator> healthIndicators;

	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
		reportHealth(request, response);
	}

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		reportHealth(request, response);
	}

	private void reportHealth(HttpServletRequest request, HttpServletResponse response) throws IOException {
		response.setHeader("Content-Type", "application/json; charset=UTF-8");
		response.setCharacterEncoding("UTF-8");

		int status = 200;
		boolean isFirst = true;
		StringBuilder result = new StringBuilder();

		if (healthIndicators != null) {
			for (HealthIndicator healthIndicator : healthIndicators) {
				HealthIndication healthIndication = healthIndicator.getHealthInfo();
				if (healthIndication.getStatus() > status) {
					status = healthIndication.getStatus();
				}
				if (!isFirst) {
					result.append(", ");
				} else {
					isFirst = false;
				}
				result.append(healthIndication.getText());
			}
		}

		PrintWriter out = response.getWriter();
		out.println("{" + result + "}");
		response.setStatus(status);
	}
}